import java.awt.GridLayout;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedWriter;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Random;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

/**
 * Panel presente en teoricoFrame utilizado para organizar las alternativas de una pregunta del quiz teorico
 * junto a sus respectivo checkbox seleccionable.
 * @author Carlos Apablaza Brito & Rodrigo Mujica Ravest
 */
public class alternativesPanel extends JPanel{
	public Quiz quiz;
	public String username = null;
	public imgPanel ipanel;
	public JCheckBox optionA;
	public JCheckBox optionB;
	public JCheckBox optionC;
	public JCheckBox optionD;
	public JLabel numberans = new JLabel();
	public int qNum = 1;
	public GridLayout altsGrid = new GridLayout(5,1);

	  /**
	    * Constructor para dar forma al panel basico.
	    * @param pregs Objeto con un quiz ya generado aleatoriamente.
	    * @param imgpanel Panel necesario para la utilizacion de imagenes.
	    * @param user Usiario realizando la prueba.
	    */
	public alternativesPanel(Quiz pregs, imgPanel imgpanel, String user){
		quiz = pregs;
		username = user;
		ipanel = imgpanel;
		this.setLayout(altsGrid);
		numberans.setText("Marque " + quiz.questions[qNum-1].nAnss + " respuesta");
		optionA = new JCheckBox(quiz.questions[qNum-1].alternatives[0]);
		optionB = new JCheckBox(quiz.questions[qNum-1].alternatives[1]);
		optionC = new JCheckBox(quiz.questions[qNum-1].alternatives[2]);
		optionD = new JCheckBox(quiz.questions[qNum-1].alternatives[3]);
		if(quiz.questions[qNum-1].hasPic){
			 ipanel.setImage(quiz.questions[qNum-1].number);
		}
		add(numberans);
		add(optionA);
		add(optionB);
		add(optionC);
		add(optionD);
	}
	
	 /**
	    * Funcion para obtener que alternativas han sido marcadas.
	    * @return retorna el valor numerico entre 1 y 4 para las alternativas marcadas de a) a d) respectivamente
	    */
	public int[] getAnswers(){
		int n = quiz.questions[qNum-1].nAnss;
		int aux[] = new int[n];
		int i=0;
		if(optionA.isSelected()) {
			aux[i]=1; 
			i++;
		}
		if(optionB.isSelected()) {
			i++;
			if(i>n){
				aux[0]=0;
				return aux;
			}
			aux[i-1]=2;	
		}
		if(optionC.isSelected()) {
			i++;
			if(i>n){
				aux[0]=0;
				return aux;
			}
			aux[i-1]=3; 
		}
		if(optionD.isSelected()) {
			i++;
			if(i>n){
				aux[0]=0;
				return aux;
			}
			aux[i-1]=4; 
		}
		return aux;
	}
	
	 /**
	    * Realiza los cambios en el panel para pasar a la siguiente pregunta.
	    */
	public void next(){
		removeAll();
		qNum++;
		numberans.setText("Marque " + quiz.questions[qNum-1].nAnss + " respuesta(s).");
		optionA = new JCheckBox(quiz.questions[qNum-1].alternatives[0]);
		optionB = new JCheckBox(quiz.questions[qNum-1].alternatives[1]);
		optionC = new JCheckBox(quiz.questions[qNum-1].alternatives[2]);
		optionD = new JCheckBox(quiz.questions[qNum-1].alternatives[3]);
		add(numberans);
		add(optionA);
		add(optionB);
		add(optionC);
		add(optionD);
		this.paintAll(getGraphics());
	}
	
	 /**
	    * Mostrar en el panel, el numero de aciertos, desaciertos y tiempo utilizados para
	    * finalizar el quiz.
	    * @param correct numero de preguntas correctamente contestadas
	    * @param wrong numero de preguntas erroneamente contestadas
	    * @param time tiempo utilizado para finalizar el quiz.
	    */
	public void showResults(int correct, int wrong, String time){
		removeAll();
		String aux[] = time.split(":");
		int totalmins = 19-Integer.parseInt(aux[0]);
		int totalsecs = 60-Integer.parseInt(aux[1]);
		JLabel results1 = new JLabel("Tiene " +correct+ " respuesta(s) correcta(s) y " +wrong+ " mala(s).");
		JLabel results2 = new JLabel("Se demoro " + totalmins + ":" + totalsecs + " en completar el Quiz.");
		add(results1);
		add(results2);
		this.setSize(500, 200);
		saveResults(correct);
		this.paintAll(getGraphics());
	}
	
	 /**
	    * Almacenar en el archivo "user"_teorico.txt el resultado del quiz recien tomado.
	    * @param correct numero de preguntas correctamente contestadas
	    */
	private void saveResults(int correct) {
		File infile;
		File outfile;
		
		infile = new File("profiles\\"+ username +"_teorico.txt");
		outfile = new File("profiles\\temp.txt");
		addDataToFile(infile, outfile, correct);
		rearrangeFile(outfile,infile);
	}

	/**
	    * Copia el contenido de "in" en el archivo "out", utilizado en el momento de la 
	    * actualizacion de datos de una prueba
	    * @param in archivo desde el cual se leera el contenido
	    * @param out archivo al cual se copiara el contenido
	    */
	private void rearrangeFile(File in, File out) {
		FileInputStream fis = null;
		BufferedInputStream bis = null;
		DataInputStream dis = null;
		FileWriter fout = null;
		PrintWriter fileout = null;
		boolean isFirstLine = true;
		
		try {
			fout = new FileWriter(out);
			fileout = new PrintWriter(fout);
			fis = new FileInputStream(in);
			bis = new BufferedInputStream(fis);
			dis = new DataInputStream(bis);
			
			while(dis.available()>0){
				fileout.println(dis.readLine());
	            fileout.flush();
			}
			
			fis.close();
			bis.close();
			dis.close();
			fout.close();
			fileout.close();
		  } catch (FileNotFoundException e) {
			      e.printStackTrace();
		  } catch (IOException e) {
			      e.printStackTrace();
		  }		
		
	}

	/**
	    * Copia el contenido de "in" en "out", con la diferencia de que aumenta el valor 
	    * escrito en la primera linea del archivo y a la salida escribe una linea mas
	    * con el valor obtenido en la prueba recien finalizada.
	    * @param in archivo desde el cual se leera el contenido
	    * @param out archivo al cual se copiara el contenido
	    * @param correct numero de respuestas correctas
	    */
	private void addDataToFile(File in, File out, int correct) {
		FileInputStream fis = null;
		BufferedInputStream bis = null;
		DataInputStream dis = null;
		FileWriter fout = null;
		PrintWriter fileout = null;
		boolean isFirstLine = true;
		
		try {
			fout = new FileWriter(out);
			fileout = new PrintWriter(fout);
			fis = new FileInputStream(in);
			bis = new BufferedInputStream(fis);
			dis = new DataInputStream(bis);
			
			while(dis.available()>0){
				if(isFirstLine){
					int a = Integer.parseInt(dis.readLine());
					fileout.println(a+1);
					isFirstLine = false;
				}
				else
					fileout.println(dis.readLine());
	            fileout.flush();
			}
			fileout.println(correct);
			fileout.flush();
			
			fis.close();
			bis.close();
			dis.close();
			fout.close();
			fileout.close();
			

		  } catch (FileNotFoundException e) {
			      e.printStackTrace();
		  } catch (IOException e) {
			      e.printStackTrace();
		  }		
		
	}

}
