import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.Timer;

/**
 * Panel presente en teoricoFrame, con la funcion de permitir al usuario pasar a la siguiente
 * pregunta y ademas avisa del tiempo restante de la prueba en curso.
 * @author Carlos Apablaza Brito & Rodrigo Mujica Ravest
 */
public class controlPanel extends JPanel implements ActionListener {
	public enunciatePanel epanel;
	public alternativesPanel apanel;
	public imgPanel ipanel;
	public int correct = 0;
	public int wrong = 0;
	public Quiz quiz;
	private boolean finish = false;
	final JLabel time;
	JLabel space;
	GridLayout altsGrid = new GridLayout(1,3);
	JButton next = new JButton("Next");
	
	/**
	    * Constructor basico.
	    * @param enpanel Panel de enunciado tambien presente en teoricoFrame.
	    * @param altspanel Panel de alternativas tambien presente en teoricoFrame.
	    * @param imgpanel Panel de imagen tambien presente en teoricoFrame.
	    * @param q Quiz siendo tomado enla prueba actual.
	    */
	public controlPanel(enunciatePanel enpanel, alternativesPanel altspanel, imgPanel imgpanel, Quiz q){
		epanel = enpanel;
		apanel = altspanel;
		ipanel = imgpanel;
		quiz = q;
		time = new JLabel("20:00");
		space = new JLabel("          Tiempo Restante: ");
		next.addActionListener(this);
		add(next);
		add(space);
		add(time);
		RelojModeloSwing modelo = new RelojModeloSwing(this);
        modelo.addObserver(new Observer()
        {
            public void update (Observable unObservable, Object dato)
            {
                if(!finish)
                	time.setText(String.valueOf(dato));
            }
        });
	}
	
	/**
	  * Controlador del evento de hacer click en el boton Next del panel. Debe generar la 
	  * interfaz de la nueva pregunta en el mismo frame.
	  * @param e Evento de realizar click sobre el boton Next.
	  */
	 public void actionPerformed(ActionEvent e) {
		 if(!finish){
			 int a[] = apanel.getAnswers();
			 int i;
			 checkAns(a);
			 if(epanel.qNum<quiz.nmax){
				 epanel.next();
				 apanel.next();
				 if(quiz.questions[epanel.qNum-1].hasPic){
					 ipanel.setImage(quiz.questions[epanel.qNum-1].number);
				 }
				 else
					ipanel.clean();
			 }
			 else{
				 epanel.clean();
				 ipanel.clean();
				 apanel.showResults(correct,wrong,time.getText());
				 removeAll();
				 repaint();
				 finish = true;
			 }
		 }
		 else{
		 }
	 }
	 
	 /**
	  * Comprueba si el dato de entrada es la respuesta correcta de la pregunta.
	  * @param a[] Arreglo con las alternativas marcadas en la interfaz (puede ser mas de 1 o ninguna).
	  */
	private void checkAns(int a[]){
		 int i;
		 for(i=0;i<a.length;i++){
			 if(a[i]==quiz.questions[epanel.qNum-1].ans[i]){}
			 else{
				 wrong++;
				 return;
			 }
		 }
		 correct++;
		 }
	
	/**
	  * Finaliza el examen teorico en el estado enq ue se encuentre. Usada en caso de que
	  * se temrine el tiempo predeterminado para la prueba
	  */
	public void finishNow(){
		 epanel.clean();
		 ipanel.clean();
		 apanel.showResults(correct,quiz.nmax-correct,time.getText());
		 removeAll();
		 repaint();
		 finish = true;
	}
	 }
/**
 * Clase temporizadora utilizada para llevar la cuenta del tiempo tomado en la prueba actual.
 * @author Carlos Apablaza Brito & Rodrigo Mujica Ravest
 */
class RelojModeloSwing extends Observable{
	private Date inittime = new Date();
	SimpleDateFormat format = new SimpleDateFormat("mm:ss");
	controlPanel cpanel = null;
	/**
	 * Constructor basico. Lanza un timer cada segundo, avisando a los observadores de este
	 * modelo del cambio.
	 * @param ctrl Panel de control que despliega la informacion del timer. 
	 */
public RelojModeloSwing(controlPanel ctrl)
{
	cpanel = ctrl;
    Timer timer = new Timer (1000, new ActionListener ()
    {
    	/**
    	 * Formatea el tiempo actual llevado en la prueba y avisa a los observadores de esta
    	 * cifra.
    	 * @param e Evento de un click en el timer (1[s]) 
    	 */
        public void actionPerformed(ActionEvent e)
        {
            setChanged();
            Date diff = getTime();
            String time = format.format(diff);
            notifyObservers (time);
        }
        /**
    	 * Obtiene la diferencia de tiempo entre el momento en que se inicio la prueba
    	 * y el actual, delegando este valor a los observadores.
    	 * @return Entrega la fecha modificada que representa la cuenta regresiva. 
    	 */
		private Date getTime() {
			Date aux = new Date();
			aux.setMinutes(19-(aux.getMinutes()-inittime.getMinutes()));
            aux.setSeconds(60-(aux.getSeconds()-inittime.getSeconds()));
            if((aux.getSeconds()==0) && (aux.getMinutes()==0)){
            	cpanel.finishNow();
            }
            	
			return aux;
		}
    });
    timer.start();
}
}
